import hashlib
import random

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.db import models
from django.template.loader import render_to_string
from django.template import TemplateDoesNotExist

from .utils import send_email

# app settings
SIGNUP_ACTIVATION_DAYS = getattr(settings, 'SIGNUP_ACTIVATION_DAYS', 2)
SIGNUP_FORM_CLASS = getattr(settings, 'SIGNUP_FORM_CLASS', None)

class ValidationManager(models.Manager):
	def create_validation(self, user):
		v = Validation(user=user)

		salt = hashlib.sha256(str(random.random())).hexdigest()
		key = hashlib.sha256(salt + user.get_full_name()).hexdigest()

		v.key = key
		v.save(using=self._db)
		return v
	
	def create_inactive_user(self, **kwargs):
		UserModel = get_user_model()
		fields = [UserModel.USERNAME_FIELD] + UserModel.REQUIRED_FIELDS

		# Make sure that all required fields are submitted to create_user
		user_kwargs = {key: kwargs[key] for key in fields}
		for p in ['password', 'password1', 'password2']:
			if kwargs.get(p):
				user_kwargs['password'] = kwargs[p]
				break

		user = UserModel.objects.create_user(**user_kwargs)
		user.is_active = False
		user.save(using=self._db)

		return user
	
class Validation(models.Model):
	user = models.ForeignKey(settings.AUTH_USER_MODEL, unique=True)
	key = models.CharField(max_length=64)
	created = models.DateTimeField(auto_now_add=True)

	objects = ValidationManager()

	def get_activation_url(self, site):
		url = "http://%(site)s%(path)s" % {
			'site': site,
			'path': reverse('signup_activate', kwargs={'activation_key': self.key})
		}

		return url

	def send_activation_email(self, site):
		template_name = 'registration/activation_email.txt'

		# context data
		url = self.get_activation_url(site)
		context = {
			'site': site,
			'url': url,
			'activation_days': SIGNUP_ACTIVATION_DAYS
		}

		to = self.user.email

		send_email(template_name, context, to)
	
	def activate_user(self):
		self.user.is_active = True
		self.user.save()




from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

class DefaultUserCreationForm(UserCreationForm):
	"""
	A form used to create a new user. This is only useful to the default user
	model, which has 'username', 'email' and 'password' fields.
	"""
	email = forms.EmailField(_('Email address'))

	def __init__(self, *args, **kwargs):
		super(DefaultUserCreationForm, self).__init__(*args, **kwargs)

		self.fields.keyOrder = [
			'username', 'email', 'password1', 'password2'
		]



from django.conf.urls import patterns, url, include

from .views import SignUpView, SignUpCompleteView, ActivateView, ActivateCompleteView

urlpatterns = patterns('',
	url(r'^signup/$', SignUpView.as_view(), name='signup_signup'),
	url(r'^signup/complete/$', SignUpCompleteView.as_view(), name='signup_signup_complete'),
	url(r'^activate/complete/$', ActivateCompleteView.as_view(), name='signup_activate_complete'),
	url(r'^activate/(?P<activation_key>\w+)/$', ActivateView.as_view(), name='signup_activate'),
	url(r'', include('django.contrib.auth.urls')),
)



from itertools import izip_longest
import re

from django.conf import settings
from django.template.loader import render_to_string
from django.core.mail import EmailMultiAlternatives

FILE_BOUNDARY_RE = re.compile(r'^-- title=(\w+).*$', re.MULTILINE)

# grouper recipe: http://docs.python.org/2/library/itertools.html#recipes
def grouper(iterable, n, fillvalue=None):
	"Collect data into fixed-length chunks or blocks"
	# grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
	args = [iter(iterable)] * n
	return izip_longest(fillvalue=fillvalue, *args)

def split_template_file(template_name, context):
	template = render_to_string(template_name, context)

	# split the rendered file and remove empty entries (usually the first entry
	# in the list is empty). also strip each entry of extra newlines --- they
	# improve readability in the original file but will make the end message
	# confusing
	entries = FILE_BOUNDARY_RE.split(template)
	entries = filter(None, entries)
	entries = map(lambda x: x.strip(), entries)

	template_contents = dict(grouper(entries, 2))
	return template_contents

def send_email(template_name, context, to, from_email=settings.DEFAULT_FROM_EMAIL):
	template_contents = split_template_file(template_name, context)

	subject = template_contents['subject']
	msg_txt = template_contents['txt']

	# remove subject newlines before continuing
	subject = ''.join(subject.splitlines())
	email = EmailMultiAlternatives(subject, msg_txt, from_email, [to])

	# attach a text/html part if it exists
	if 'html' in template_contents:
		email.attach_alternative(template_contents['html'], 'text/html')

	email.send()



from django.views.generic.edit import FormView
from django.views.generic.base import TemplateView
from django.core.urlresolvers import reverse_lazy
from django.contrib.auth import get_user_model
from django.contrib.sites.models import get_current_site
from django.shortcuts import redirect

from .models import Validation, SIGNUP_FORM_CLASS

class SignUpView(FormView):
	template_name = 'registration/signup_form.html'
	success_url = reverse_lazy('signup_signup_complete')

	def get_form_class(self):
		if self.form_class is None:
			form_class_name = SIGNUP_FORM_CLASS
			if form_class_name is None:
				from .forms import DefaultUserCreationForm
				self.form_class = DefaultUserCreationForm
			else:
				import importlib
				module_name, class_name = form_class_name.rsplit('.', 1)
				module = importlib.import_module(module_name)
				self.form_class = getattr(module, class_name)

		return self.form_class
	
	def form_valid(self, form):
		user = Validation.objects.create_inactive_user(**form.cleaned_data)
		validation = Validation.objects.create_validation(user)

		validation.send_activation_email(get_current_site(self.request))

		return super(SignUpView, self).form_valid(form)

		
class SignUpCompleteView(TemplateView):
	template_name = 'registration/signup_complete.html'
	
class ActivateView(TemplateView):
	template_name = 'registration/activation_failed.html'

	def get(self, request, *args, **kwargs):
		try:
			validation = Validation.objects.get(key=kwargs['activation_key'])
			validation.activate_user()
			validation.delete()
			return redirect('signup_activate_complete')
		except Validation.DoesNotExist:
			pass

		return super(ActivateView, self).get(request, *args, **kwargs)
	
class ActivateCompleteView(TemplateView):
	template_name = 'registration/activation_complete.html'




#!/usr/bin/env python

from distutils.core import setup
from signup import __version__

setup(
	name='django-signup',
	version=__version__,
	description='A user registration app for Django with support for custom user models',
	author='Felipe Bessa Coelho',
	author_email='fcoelho.9@gmail.com',
	url='http://bitbucket.org/fcoelho/django-signup/',
	packages=['signup'],
)
